// Load Mongoose OS API
load('api_sys.js');
load('api_events.js');
load('api_gpio.js');
load('api_log.js');
load('api_timer.js');
load('api_gcp.js');

// init hw to see config and generate state changes
let led_pin = 14;
let button_pin = 0;
GPIO.set_mode(led_pin, GPIO.MODE_OUTPUT);
GPIO.set_mode(button_pin, GPIO.MODE_INPUT);


// send telemetry (events)
Timer.set(5000 /* milliseconds */, Timer.REPEAT, function() {
    if(GCP.isConnected()){
        let msg=JSON.stringify({t: Timer.fmt("%F %T", Timer.now())});
        Log.print(Log.INFO, 'Send event:' + ((GCP.sendEvent(msg)) ? 'OK' : 'FAIL') + ' msg:' +  msg);
    }
}, null);

// send state change
GPIO.set_button_handler(button_pin, GPIO.PULL_NONE, GPIO.INT_EDGE_POS, 100, function() {
    if(GCP.isConnected()){
        let msg = JSON.stringify({free: Sys.free_ram(), total: Sys.total_ram()}); 
        Log.print(Log.INFO, 'Send state:' + ((GCP.sendState(msg)) ? 'OK' : 'FAIL') + ' msg:' +  msg);
    }
}, null);

// receive config changes
GCP.config(function (confdata, ud) {
    if(!confdata)
      return;
    let obj = JSON.parse(confdata) || {out1: 0};
    GPIO.write(led_pin, obj.out1);
    Log.print(Log.INFO, 'Received config: ' + confdata + ' output: ' + ((obj.out1)? 'ON':'OFF'));
}, null);

// receive commands
GCP.command(function (cmddata, subfolder, ud) {
    if(!cmddata)
      return;
    let cmd = JSON.parse(cmddata) || {};
    Log.print(Log.INFO, 'Received command: ' + cmddata);
    if(subfolder)
      Log.print(Log.INFO, 'for subfolder:: ' + subfolder);
}, null);

// handle connect/disconnect
Event.addHandler(Event.GCP_CONNECT, function (ev, evdata, ud) {
    Log.print(Log.INFO, 'Connected to GCP');
}, null);
Event.addHandler(Event.GCP_CLOSE, function (ev, evdata, ud) {
    Log.print(Log.INFO, 'Disconnected from GCP');
}, null);


// generic time change, includes SNTP (SYS+3)
Event.addHandler(Event.OTA_TIME_CHANGED, function (ev, evdata, ud) {
    Log.print(Log.INFO, 'Time set');
}, null);

// generic cloud events (others above are specific GCP events)
Event.addHandler(Event.CLOUD_CONNECTED, function (ev, evdata, ud) {
}, null);
Event.addHandler(Event.CLOUD_DISCONNECTED, function (ev, evdata, ud) {
}, null);


Log.print(Log.WARN, "### init script started ###");

